from django.db import models

from wagtail.admin.edit_handlers import (
    FieldPanel,
    MultiFieldPanel,
    StreamFieldPanel,
)
from wagtail.core.fields import RichTextField, StreamField
from wagtail.core.models import Collection, Page

from .blocks import BaseStreamBlock, FAQBlock


class HomePage(Page):
    """
    La page d'accueil du site. Elle est composée de :
        - un bloc d'accroche (hero) ;
        - un bloc de contenu.
    """

    hero = StreamField(
        BaseStreamBlock,
        verbose_name="accroche",
        blank=True,
        help_text=("Bloc de contenu qui sera mis en avant."),
    )
    body = StreamField(
        BaseStreamBlock,
        verbose_name="contenu",
        blank=True,
        help_text=("Bloc de contenu de la page d'accueil."),
    )
    featured_section_title = models.CharField(
        max_length=255,
        null=True,
        blank=True,
        verbose_name="titre de la section",
        help_text=("Titre affiché au dessus de la collection à présenter."),
    )
    featured_section_collection = models.ForeignKey(
        Collection,
        limit_choices_to=~models.Q(name__in=['Root']),
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name="collection présentée",
        help_text=("La collection de vidéos à afficher en page d'accueil."),
    )

    class Meta:
        verbose_name = "page d'accueil"
        verbose_name_plural = "pages d'accueil"

    # Wagtails attributes -----------------------------------------------------

    content_panels = Page.content_panels + [
        StreamFieldPanel('hero'),
        StreamFieldPanel('body'),
        MultiFieldPanel(
            [
                FieldPanel('featured_section_title'),
                FieldPanel('featured_section_collection'),
            ],
            heading="Section mise en avant",
        ),
    ]

    parent_page_types = ['wagtailcore.Page']


class StandardPage(Page):
    """
    Une page de contenu générique. Elle est utilisée pour la page
    « À propos », ainsi que toutes les pages qui n'ont besoin que de :
        - un bloc de contenu.
    """

    body = StreamField(
        BaseStreamBlock,
        verbose_name="contenu",
        help_text=("Bloc de contenu de la page."),
    )

    class Meta:
        verbose_name = "page standard"
        verbose_name_plural = "pages standards"

    # Wagtails attributes -----------------------------------------------------

    content_panels = Page.content_panels + [StreamFieldPanel('body')]


class FAQPage(Page):
    """
    Une page de foire aux questions. Elles peuvent être indépendantes ou
    regroupées dans une page `FAQIndexPage`. Cela peut permettre de séparer
    les questions par catégories par exemple.
    """

    body = StreamField(
        [('faq', FAQBlock())], blank=True, verbose_name="contenu"
    )

    class Meta:
        verbose_name = "page de foire aux questions"
        verbose_name_plural = "pages de foire aux questions"

    # Wagtails attributes -----------------------------------------------------

    content_panels = Page.content_panels + [StreamFieldPanel('body')]

    subpage_types = []

    # Routes & views ----------------------------------------------------------

    def get_context(self, request):
        context = super().get_context(request)
        # ajoute la page d'index éventuelle
        parent_page = self.get_parent()
        if parent_page.specific_class == FAQIndexPage:
            context['index_page'] = parent_page
        return context


class FAQIndexPage(Page):
    """
    Une page d'index pour des foires aux questions. Toutes les questions des
    pages `FAQPage` enfants seront présentées sur cette page.
    """

    introduction = RichTextField(
        verbose_name="introduction",
        blank=True,
        help_text=("Texte d'introduction affiché au dessus des questions."),
    )

    class Meta:
        verbose_name = "page d'index de foires aux questions"
        verbose_name_plural = "pages d'index de foires aux questions"

    # Wagtails attributes -----------------------------------------------------

    content_panels = Page.content_panels + [FieldPanel('introduction')]

    subpage_types = ['FAQPage']

    # Routes & views ----------------------------------------------------------

    def get_context(self, request):
        context = super().get_context(request)
        # ajoute les foires aux questions au contexte
        context['faq_pages'] = FAQPage.objects.descendant_of(self).live()
        return context
