from django.utils.html import format_html

from draftjs_exporter.dom import DOM
from wagtail.admin.menu import MenuItem
from wagtail.admin.rich_text.converters.html_to_contentstate import (
    BlockElementHandler,
)
from wagtail.admin.rich_text.editors.draftail.features import BlockFeature
from wagtail.core import hooks

from .templatetags.minified import minified


class SiteMenuItem(MenuItem):
    """
    Élément de menu pour retourner vers le site.
    """

    def __init__(self, label="Retour au site", url='', **kwargs):
        super().__init__(label, '', **kwargs)

    def is_active(self, *args, **kwargs):
        return False

    def get_context(self, request):
        context = super().get_context(request)
        context['url'] = request.META['SCRIPT_NAME'] or '/'
        return context


@hooks.register('register_admin_menu_item')
def register_site_menu_item():
    """
    Ajoute un élément dans le menu de l'admin pour retourner vers le site.
    """
    return SiteMenuItem(classnames='icon icon-home', order=10000)


@hooks.register('insert_global_admin_css')
def global_admin_css():
    """
    Ajoute une feuille de styles personnalisée dans l'admin.
    """
    return format_html(
        '<link rel="stylesheet" href="{}">', minified('css/admin.css')
    )


@hooks.register('register_rich_text_features')
def register_lead_feature(features):
    """
    Enregistre une fonctionnalité dans l'éditeur de texte pour agrandir la
    taille d'un paragraphe.
    """
    feature_name = 'lead-paragraph'
    type_name = 'lead'

    features.register_editor_plugin(
        'draftail',
        feature_name,
        BlockFeature(
            {
                'type': type_name,
                'icon': 'title',
                'description': "Grand paragraphe",
            }
        ),
    )
    features.register_converter_rule(
        'contentstate',
        feature_name,
        {
            'from_database_format': {
                'p[class="lead"]': BlockElementHandler(type_name)
            },
            'to_database_format': {
                'block_map': {
                    type_name: lambda p: DOM.create_element(
                        'p', {'class': 'lead'}, p['children']
                    )
                }
            },
        },
    )
    # ajoute cette fonctionnalité par défaut
    features.default_features.append(feature_name)
