import logging
import smtplib

from django import forms
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string

from jcpml.utils.forms import CustomTapeformMixin

from .models import Appel, Participant, Question, QuestionStatus

logger = logging.getLogger(__name__)


class ParticipantForm(CustomTapeformMixin, forms.ModelForm):
    """
    Formulaire de création ou d'édition d'un participant.
    """

    genre = forms.ChoiceField(
        label="Vous êtes…",
        choices=Participant.GENRE_CHOICES,
        widget=forms.RadioSelect,
    )

    field_template_overrides = {
        'ville': 'forms/fields/without_label.html',
        'ville_autre': 'forms/fields/without_label.html',
    }

    class Meta:
        model = Participant
        fields = ['genre', 'age', 'ville', 'ville_autre']
        widgets = {
            'ville_autre': forms.TextInput(attrs={'placeholder': "Précisez…"})
        }


class QuestionAddForm(ParticipantForm):
    """
    Formulaire d'ajout d'une question par un participant.
    """

    layout_template = 'forum/forms/layouts/question.html'

    class Meta:
        model = Question
        fields = ParticipantForm.Meta.fields + ['question', 'email']
        widgets = dict(
            ParticipantForm.Meta.widgets,
            question=forms.Textarea(attrs={'cols': None, 'rows': 3}),
        )


class AppelAddForm(ParticipantForm):
    """
    Formulaire d'ajout d'une demande d'appel d'un participant.
    """

    layout_template = 'forum/forms/layouts/appel.html'

    class Meta:
        model = Appel
        fields = ParticipantForm.Meta.fields + ['objet', 'telephone']
        widgets = dict(
            ParticipantForm.Meta.widgets,
            objet=forms.Textarea(attrs={'cols': None, 'rows': 3}),
        )


class QuestionAnswerForm(CustomTapeformMixin, forms.ModelForm):
    """
    Formulaire de réponse à une question.
    """

    answer = forms.CharField(
        label="Réponse",
        required=True,
        min_length=10,
        widget=forms.Textarea(attrs={'rows': 4}),
    )

    class Meta:
        model = Question
        fields = ['answer']

    def save(self, *args, **kwargs):
        self.instance.status = QuestionStatus.traite
        return super().save(*args, **kwargs)

    def send_email(self):
        """Envoi par courriel la réponse à la personne."""
        context = {
            'subject': "Réponse à votre question",
            'question': self.instance,
        }
        to_email = self.instance.email
        from_email = settings.DEFAULT_FROM_EMAIL
        message_txt = render_to_string(
            'forum/question/reponse_email.txt', context
        )
        message_html = render_to_string(
            'forum/question/reponse_email.html', context
        )

        # construit et envoi le courriel
        email = EmailMultiAlternatives(
            context['subject'], message_txt, from_email, [to_email]
        )
        email.attach_alternative(message_html, 'text/html')
        try:
            email.send()
        except smtplib.SMTPException:
            logger.exception("Erreur lors de l'envoi du mail")
            return False
        return True


class AppelUpdateForm(CustomTapeformMixin, forms.ModelForm):
    """
    Formulaire de modification d'une demande d'appel d'un participant.
    """

    class Meta:
        model = Appel
        fields = ['status', 'notes']
