from django.core.exceptions import ValidationError
from django.db import models

from djchoices import ChoiceItem, DjangoChoices
from wagtail.contrib.settings.models import BaseSetting, register_setting

from .choices import VILLE_CHOICES


@register_setting
class ForumSettings(BaseSetting):
    is_active = models.BooleanField(
        "actif",
        default=False,
        help_text=(
            "Détermine si le forum est disponible pour les visiteurs. Cela "
            "contrôle la possibilité de poser une question par exemple."
        ),
    )

    class Meta:
        verbose_name = "Forum virtuel"


class Participant(models.Model):
    """
    Une personne participant au forum.
    """

    GENRE_CHOICES = (('F', "Une femme"), ('H', "Un homme"))
    AGE_CHOICES = (
        ('18-20', "18 à 20 ans"),
        ('21-24', "21 à 24 ans"),
        ('25-30', "25 à 30 ans"),
        ('30+', "plus de 30 ans"),
    )
    VILLE_AUTRE = 'autre'

    genre = models.CharField("genre", max_length=1, choices=GENRE_CHOICES)
    age = models.CharField("âge", max_length=5, choices=AGE_CHOICES)
    ville = models.CharField(
        "ville de résidence",
        max_length=30,
        choices=VILLE_CHOICES + [(VILLE_AUTRE, "Autre...")],
    )
    ville_autre = models.CharField(
        "autre ville de résidence", max_length=30, blank=True
    )
    session_id = models.CharField(
        "id de session", max_length=64, blank=True, editable=False
    )

    class Meta:
        verbose_name = "participant(e)"
        verbose_name_plural = "participant(e)s"

    def get_ville_residence_display(self):
        """Retourne la ville de résidence à afficher."""
        if self.ville == self.VILLE_AUTRE:
            return "Autre : {:s}".format(self.ville_autre)
        else:
            return self.get_ville_display()

    def clean(self):
        if self.ville == self.VILLE_AUTRE and not self.ville_autre:
            raise ValidationError(
                {
                    'ville_autre': (
                        "Veuillez remplir ce champ si votre ville de résidence "
                        "ne figure pas dans la liste."
                    )
                }
            )
        super().clean()


class History(models.Model):
    """
    Un historique de création et de modification d'un objet.
    """

    created_at = models.DateTimeField("date de création", auto_now_add=True)
    last_modified_at = models.DateTimeField(
        "date de la dernière modification", auto_now=True
    )

    class Meta:
        abstract = True
        ordering = ['last_modified_at']


# QUESTION POSÉE PAR UN(E) PARTICIPANT(E)
# -----------------------------------------------------------------------------


class QuestionStatus(DjangoChoices):
    """
    Les statuts d'une question posée.
    """

    nouveau = ChoiceItem('nouveau', "Nouveau")
    en_cours = ChoiceItem('en_cours', "En cours")
    traite = ChoiceItem('traite', "Traité")
    prive = ChoiceItem('prive', "Privé")
    spam = ChoiceItem('spam', "Spam")


class QuestionManager(models.Manager):
    def a_traiter(self):
        """Filtre avec les questions à traiter uniquement."""
        return self.filter(status=QuestionStatus.nouveau)

    def en_cours(self):
        """Filtre avec les questions en cours uniquement."""
        return self.filter(status=QuestionStatus.en_cours)


class Question(History, Participant):
    """
    Une question posée par un(e) participant(e).
    """

    status = models.CharField(
        "statut",
        max_length=10,
        choices=QuestionStatus.choices,
        default=QuestionStatus.nouveau,
    )
    email = models.EmailField("adresse mail")
    question = models.TextField("question")
    answer = models.TextField("réponse", blank=True)
    participant_ptr = models.OneToOneField(
        Participant,
        on_delete=models.CASCADE,
        parent_link=True,
        related_name='question_ptr',
    )

    objects = QuestionManager()


# DEMANDE D'APPEL D'UN(E) PARTICIPANT(E)
# -----------------------------------------------------------------------------


class AppelStatus(DjangoChoices):
    """
    Les statuts d'une demande d'appel.
    """

    nouveau = ChoiceItem('nouveau', "Nouveau")
    en_cours = ChoiceItem('en_cours', "En cours")
    traite = ChoiceItem('traite', "Traité")
    rappeler = ChoiceItem('rappeler', "À rappeler")
    erreur = ChoiceItem('erreur', "Injoignable")


class AppelManager(models.Manager):
    def a_traiter(self):
        """Filtre avec les appels à traiter uniquement."""
        return self.filter(
            status__in=[AppelStatus.nouveau, AppelStatus.rappeler]
        )

    def en_cours(self):
        """Filtre avec les appels en cours uniquement."""
        return self.filter(status=AppelStatus.en_cours)


class Appel(History, Participant):
    """
    Une demande d'appel d'un(e) participant(e).
    """

    status = models.CharField(
        "statut",
        max_length=10,
        choices=AppelStatus.choices,
        default=AppelStatus.nouveau,
    )
    telephone = models.CharField("téléphone", max_length=15)
    objet = models.TextField("objet de la demande")
    notes = models.TextField("notes", blank=True)
    compteur = models.PositiveSmallIntegerField(
        "nombre de tentatives", default=0, editable=False
    )

    objects = AppelManager()
