from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse_lazy
from django.views.generic import DetailView, FormView, TemplateView, UpdateView

from .forms import (
    AppelAddForm,
    AppelUpdateForm,
    QuestionAddForm,
    QuestionAnswerForm,
)
from .models import Appel, AppelStatus, ForumSettings, Question, QuestionStatus

# VUES PUBLIQUES
# -----------------------------------------------------------------------------


class ActiveForumMixin:
    """
    Mixin pour empêcher l'accès à la vue si le forum n'est pas activé.
    """

    def dispatch(self, request, *args, **kwargs):
        forum_settings = ForumSettings.for_site(request.site)
        if not forum_settings.is_active and not request.user.is_authenticated:
            messages.error(
                request,
                ("Le forum est actuellement désactivé, revenez plus tard !"),
            )
            return HttpResponseRedirect(request.META['SCRIPT_NAME'] or '/')
        return super().dispatch(request, *args, **kwargs)


class ParticipantAddView(FormView):
    ajax_template_name = None

    def dispatch(self, request, *args, **kwargs):
        self.object = None
        return super().dispatch(request, *args, **kwargs)

    def get_template_names(self):
        """Retourne le template à utiliser pour la requête."""
        if self.ajax_template_name and self.request.is_ajax():
            return [self.ajax_template_name]
        return super().get_template_names()

    def form_valid(self, form):
        """Enregistre l'objet et retourne la réponse."""
        self.object = form.save(commit=False)
        if self.request.session.session_key is None:
            # initialise la session du participant
            self.request.session['is_participant'] = True
        else:
            # ou stocke l'identifiant de sa session
            self.object.session_id = self.request.session.session_key
        self.object.save()
        # retourne le même template avec l'objet uniquement
        return self.render_to_response({'object': self.object})


class QuestionAddView(ActiveForumMixin, ParticipantAddView):
    form_class = QuestionAddForm
    template_name = 'forum/question/add.html'
    ajax_template_name = 'forum/question/add_form.html'


class AppelAddView(ActiveForumMixin, ParticipantAddView):
    form_class = AppelAddForm
    template_name = 'forum/appel/add.html'
    ajax_template_name = 'forum/appel/add_form.html'


# ADMINISTRATION DES QUESTIONS
# -----------------------------------------------------------------------------


@login_required
def question_reset(request, pk):
    """Réinitialise le statut d'une question."""
    question = get_object_or_404(Question, pk=pk)
    question.status = QuestionStatus.nouveau
    question.save()
    return redirect('forum:question-files')


@login_required
def question_spam(request, pk):
    """Marque une question comme spam."""
    question = get_object_or_404(Question, pk=pk)
    question.status = QuestionStatus.spam
    question.save()
    messages.success(
        request,
        ("La question {} a été marquée comme spam.".format(question.pk)),
    )
    return redirect('forum:question-files')


class QuestionAnswerView(LoginRequiredMixin, UpdateView):
    """Répond à une nouvelle question."""

    model = Question
    form_class = QuestionAnswerForm
    success_url = reverse_lazy('forum:question-files')
    template_name = 'forum/question/answer.html'

    def get_object(self, *args, **kwargs):
        # change le statut de la question
        obj = super().get_object(*args, **kwargs)
        obj.status = QuestionStatus.en_cours
        obj.save()
        return obj

    def form_valid(self, form):
        response = super().form_valid(form)
        if not form.send_email():
            messages.error(
                self.request,
                ("Une erreur est survenue lors de l'envoi du mail."),
            )
        else:
            messages.success(
                self.request, ("Votre réponse a été envoyée à la personne.")
            )
        return response


class QuestionFilesView(LoginRequiredMixin, TemplateView):
    """Liste l'ensemble des questions à traiter ou en cours."""

    template_name = 'forum/question/files.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['questions'] = {
            'a_traiter': Question.objects.a_traiter(),
            'en_cours': Question.objects.en_cours(),
        }
        return context


# ADMINISTRATION DES APPELS
# -----------------------------------------------------------------------------


@login_required
def appel_process(request, pk):
    """Passe le statut d'un appel à « en cours »."""
    appel = get_object_or_404(Appel, pk=pk)
    appel.status = AppelStatus.en_cours
    appel.compteur += 1
    appel.save()
    return redirect('forum:appel-files')


class AppelDetailView(LoginRequiredMixin, DetailView):
    """Affiche les détails d'un appel."""

    model = Appel
    template_name = 'forum/appel/detail.html'


class AppelUpdateView(LoginRequiredMixin, UpdateView):
    """Mets à jour un appel."""

    model = Appel
    form_class = AppelUpdateForm
    success_url = reverse_lazy('forum:appel-files')
    template_name = 'forum/appel/update.html'

    def form_valid(self, form):
        response = super().form_valid(form)
        messages.success(
            self.request,
            (
                "La demande d'appel {} a été mise à jour.".format(
                    self.object.pk
                )
            ),
        )
        return response


class AppelFilesView(LoginRequiredMixin, TemplateView):
    """Liste l'ensemble des appels à traiter ou en cours."""

    template_name = 'forum/appel/files.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['appels'] = {
            'a_traiter': Appel.objects.a_traiter(),
            'en_cours': Appel.objects.en_cours(),
        }
        return context
