import mimetypes
import os.path
from argparse import FileType

from django.core.management.base import BaseCommand, CommandError

from jcpml.media.models import SourceMedia, VideoSource
from jcpml.media.utils import get_media_info


class Command(BaseCommand):
    help = "Ajoute une ou plusieurs sources vidéo pour un média"

    def add_arguments(self, parser):
        # arguments requis
        parser.add_argument(
            'media_id',
            type=int,
            metavar='MEDIA_ID',
            help="Identifiant du média",
        )
        parser.add_argument(
            'sources',
            nargs='+',
            type=FileType('rb'),
            metavar='SOURCE',
            help="Fichier vidéo à ajouter",
        )

    def handle(self, **options):
        try:
            media = SourceMedia.objects.get(pk=options['media_id'])
        except SourceMedia.DoesNotExist:
            raise CommandError(
                "Le media {0} n'existe pas".format(options['media_id'])
            )

        debug = self.get_debugger(int(options['verbosity']))

        for source in options['sources']:
            debug(":: traitement de : {}", source.name)

            # devine et valide le type de vidéo
            mime_type = mimetypes.guess_type(source.name)[0]
            if not mime_type:
                self.stderr.write(
                    self.style.ERROR(
                        "Impossible de deviner le type MIME de {}".format(
                            source.name
                        )
                    )
                )
                continue
            if not mime_type.startswith('video/'):
                self.stderr.write(
                    self.style.ERROR(
                        "Le fichier {} ne semble pas être une vidéo".format(
                            source.name
                        )
                    )
                )
                continue

            # récupère la résolution de la vidéo
            media_info = get_media_info(source.name)
            if not media_info or not media_info['streams']['video']:
                self.stderr.write(
                    self.style.ERROR(
                        "Impossible de récupérer les informations sur le flux "
                        "vidéo de {}".format(source.name)
                    )
                )
                continue

            # crée et enregistre la nouvelle source vidéo
            src = VideoSource(media=media, mime_type=mime_type)
            src.width = media_info['streams']['video'][0]['width']
            src.height = media_info['streams']['video'][0]['height']
            src.file.save(os.path.basename(source.name), source)

            debug(" mime_type = {}", src.mime_type)
            debug(" resolution = {}x{}", src.width, src.height)

            self.stderr.write(
                self.style.SUCCESS(
                    "Le fichier {0} a été ajouté aux sources vidéos de "
                    "« {1} »".format(source.name, media.title)
                )
            )

    def get_debugger(self, verbosity):
        if verbosity > 1:

            def debugger(msg, *args, **kwargs):
                self.stdout.write(msg.format(*args, **kwargs))

        else:

            def debugger(msg, *args, **kwargs):
                pass

        return debugger
