from django.db import models
from django.forms.utils import flatatt
from django.urls import reverse
from django.utils.safestring import mark_safe

from wagtailmedia.models import AbstractMedia


class SourceTagMixin:
    """
    Génère le code HTML pour une balise <source>. Le modèle peut redéfinir
    la propriété `source_attrs_dict` ou se contenter de fournir les
    propriétés `url` et `mime_type`.
    """

    @property
    def source_attrs(self):
        """
        Le code HTML des attributs de cette source pour une balise <source>.
        """
        return flatatt(self.source_attrs_dict)

    @property
    def source_attrs_dict(self):
        """
        Un dictionnaire des attributs de cette source pour une balise <source>.
        """
        attrs = {'src': self.url}
        if getattr(self, 'mime_type', None):
            attrs['type'] = self.mime_type
        if getattr(self, 'height', None):
            attrs['size'] = self.height
        return attrs

    def source_tag(self, extra_attributes={}):
        attrs = self.source_attrs_dict.copy()
        attrs.update(extra_attributes)
        return mark_safe('<source{}>'.format(flatatt(attrs)))


class SourceMedia(SourceTagMixin, AbstractMedia):
    """
    Un média source de différents types (cf. ``AbstractMedia.MEDIA_TYPES``).
    """

    duration = models.PositiveIntegerField(
        verbose_name="durée",
        blank=True,
        null=True,
        help_text=("Durée du média en secondes."),
    )
    mime_type = models.CharField(
        max_length=12, blank=True, null=True, editable=False
    )

    admin_form_fields = ('title', 'file', 'collection', 'thumbnail', 'tags')

    def get_absolute_url(self):
        return reverse('media:show', args=[str(self.id)])


class VideoSource(SourceTagMixin, models.Model):
    """
    Une source vidéo d'un média, d'un type et d'une taille donnés.
    """

    file = models.FileField(upload_to='media_sources')
    width = models.PositiveSmallIntegerField()
    height = models.PositiveSmallIntegerField()
    mime_type = models.CharField(max_length=12)

    media = models.ForeignKey(
        SourceMedia,
        related_name='video_sources',
        on_delete=models.CASCADE,
        limit_choices_to={'type': 'video'},
    )

    class Meta:
        unique_together = (('media', 'mime_type', 'height'),)

    @property
    def url(self):
        return self.file.url
